﻿using Com.Opticon.Scannersdk.Scanner;

namespace ScannerMauiTest
{
    public partial class MainPage : ContentPage
    {
        private readonly IScannerService _scannerService;

        public MainPage() : this(DependencyService.Get<IScannerService>()) { }

        public MainPage(IScannerService scannerService)
        {

            InitializeComponent();

            AppDomain.CurrentDomain.UnhandledException += (sender, e) =>
            {
                System.Diagnostics.Debug.WriteLine(e.ExceptionObject.ToString());
            };

            // Handle unobserved task exceptions
            TaskScheduler.UnobservedTaskException += (sender, e) =>
            {
                System.Diagnostics.Debug.WriteLine(e.Exception.ToString());
            };

            _scannerService = scannerService;
            _scannerService.BarcodeEvent += OnBarcodeEvent;
            _scannerService.ImageDataEvent += OnImageDataEvent;
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();
            _scannerService.AddBarcodeEventListener((IBarcodeEventListener)_scannerService);
            _scannerService.Init();
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();
            _scannerService.Deinit();
            _scannerService.RemoveBarcodeEventListener((IBarcodeEventListener)_scannerService);
        }

        private void OnBarcodeEvent(object sender, BarcodeEventArgs e)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                ScannedDataLabel.Text = e.Data;
            });
        }

        private void OnImageDataEvent(object sender, ImageDataEventArgs e)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                CapturedImage.Source = ImageSource.FromStream(() => new MemoryStream(e.Image));
            });
        }

        private void OnScanButtonClicked(object sender, EventArgs e)
        {
            if (_scannerService.IsScanning)
            {
                _scannerService.StopScan();
            }
            else
            {
                _scannerService.StartScan();
            }
        }

        private void OnToggleFloodfillButtonClicked(object sender, EventArgs e)
        {
            _scannerService.ToggleSettings();
        }

        private void OnTakePictureButtonClicked(object sender, EventArgs e)
        {
            _scannerService.TakePicture();
        }
    }

}
