﻿using Android.Graphics;
using Com.Opticon.Scannersdk.Scanner;
using Com.Opticon.Settings.Readoption;

[assembly: Dependency(typeof(ScannerMauiTest.ScannerService))]
namespace ScannerMauiTest
{
    public class ScannerService : Java.Lang.Object, IScannerService, IBarcodeEventListener
    {
        private ScannerManager scannerManager = null!;
        private IScanner scanner = null!;
        private bool isScanning;

        public event EventHandler<BarcodeEventArgs> BarcodeEvent = null!;
        public event EventHandler<ImageDataEventArgs> ImageDataEvent = null!;

        public bool IsScanning => isScanning;
        public ScannerService(IServiceProvider serviceProvider)
        {
            var context = serviceProvider.GetService<IMauiContext>();

            if (context == null)
            {
                return;
            }

#if ANDROID
            // Get the native context from the current activity
            var androidContext = context.Context;
            scannerManager = ScannerManager.GetInstance(androidContext);
#endif

            var list = scannerManager.ScannerInfoList;
            for (int i = 0; i < list.Count; i++)
            {
                ScannerInfo info = list[i];
                if (info.Type == ScannerType.SoftwareScanner)
                {
                    scanner = scannerManager.GetScanner(info);
                    break;
                }
            }
        }

        public void StartScan()
        {
            if (scanner == null || isScanning) return;
            scanner.StartScan();
        }

        public void StopScan()
        {
            if (scanner == null || !isScanning) return;
            scanner.StopScan();
        }

        public void ToggleSettings()
        {
            if (scanner == null) return;
            var settings = scanner.Settings;
            settings.ReadOption.CameraIlluminationValue = settings.ReadOption.CameraIlluminationValue == ReadOption.CameraIllumination.TurnOn
                ? ReadOption.CameraIllumination.TurnOff
                : ReadOption.CameraIllumination.TurnOn;
            scanner.SetSettings(settings);
        }

        public void TakePicture()
        {
            if (scanner == null) return;
            scanner.TakePicture(1);
        }

        public void Init()
        {
            scanner?.Init();
        }

        public void Deinit()
        {
            scanner?.Deinit();
        }

        public void AddBarcodeEventListener(IBarcodeEventListener listener)
        {
            scanner?.AddBarcodeEventListener(listener);
        }

        public void RemoveBarcodeEventListener(IBarcodeEventListener listener)
        {
            scanner?.RemoveBarcodeEventListener();
        }

        public void OnConnect()
        {
            // Handle connection logic here
        }

        public void OnDecodeStart()
        {
            isScanning = true;
        }

        public void OnDecodeStop()
        {
            isScanning = false;
        }

        public void OnDisconnect()
        {
            // Handle disconnection logic here
        }

        public void OnImageData(Bitmap bitmap, byte[] byteArray, int p2)
        {
            byte[] imageData = BitmapToByteArray(bitmap);

            Console.WriteLine($"OnImageData: imageData size = {imageData.Length}");

            if (imageData.Length > 0)
            {
                Console.WriteLine($"OnImageData: imageData first few bytes = {string.Join(",", imageData.Take(10))}");
            }

            ImageDataEvent?.Invoke(this, new ImageDataEventArgs { Image = imageData });
        }

        private byte[] BitmapToByteArray(Bitmap bitmap)
        {
            using (var stream = new MemoryStream())
            {
                bitmap.Compress(Bitmap.CompressFormat.Png, 100, stream);
                return stream.ToArray();
            }
        }

        public void OnReadData(ReadData readData)
        {
            BarcodeEvent?.Invoke(this, new BarcodeEventArgs { Data = readData.Text });
            isScanning = false;
        }

        public void OnTimeout()
        {
            // Handle timeout logic here
        }
    }
}
