﻿using Android.App;
using Android.OS;
using Android.Support.V7.App;
using Android.Runtime;
using Android.Widget;
using Com.Opticon.Scannersdk.Scanner;
using Android.Graphics;
using Com.Opticon.Settings;
using Com.Opticon.Settings.Readoption;
using System;
using static Com.Opticon.Settings.Format.Format;

namespace ScannerSdkSample_Xamarin
{
    [Activity(Label = "@string/app_name", Theme = "@style/AppTheme", MainLauncher = true)]
    public class MainActivity : AppCompatActivity ,IBarcodeEventListener
    {

        string TAG = "ScannerSdkSample_Xamarin";
        ScannerManager scannerManager;
        IScanner scanner;
        public BarcodeScannerEvents bcrScannerEvents;

        bool isScanning;

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);
            Xamarin.Essentials.Platform.Init(this, savedInstanceState);
            // Set our view from the "main" layout resource
            SetContentView(Resource.Layout.activity_main);

            scannerManager = ScannerManager.GetInstance(this);
            var list = scannerManager.ScannerInfoList;
            for (int i = 0;i<list.Count;i++)
            {
                ScannerInfo info = list[i];
                //ソフトウェアスキャナ(端末内臓スキャナ)を操作したいとき
                //When you want to control the H-35 built-in scanner
                if (info.Type == ScannerType.SoftwareScanner)
                {
                    scanner = scannerManager.GetScanner(info);
                    break;
                }
            }
            
            SetButtonEvent();

        }

        private void BcrScannerEvents_OnImageDataEvent(object sender, EventArgs e)
        {
            throw new NotImplementedException();
        }

        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, [GeneratedEnum] Android.Content.PM.Permission[] grantResults)
        {
            Xamarin.Essentials.Platform.OnRequestPermissionsResult(requestCode, permissions, grantResults);

            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);
        }


        protected override void OnResume()
        {
            base.OnResume();
            if (scanner != null)
            {
                scanner.AddBarcodeEventListener(this);
                scanner.Init();
            }
        }

        protected override void OnPause()
        {
            base.OnPause();
            if (scanner != null)
            {
                scanner.Deinit();
                scanner.RemoveBarcodeEventListener();
            }
        }


        void IBarcodeEventListener.OnReadData(ReadData readData)
        {
            //読み取りデータを取得した際に呼ばれます
            //Called when reading data is acquired
            Android.Util.Log.Debug(TAG, "onReadData:" + readData.Text);
            Android.Util.Log.Debug(TAG, "onReadData:length:" + readData.GetRawData().Length);
            Android.Util.Log.Debug(TAG, "onReadData:codeId:" + readData.CodeID);

            string data = readData.Text;
            TextView textView = FindViewById<TextView>(Resource.Id.textViewReadData);
            textView.SetText(data, TextView.BufferType.Normal);
        }

        public void OnTimeout()
        {
            //読み取りを開始し、バーコードを読み取らずに時間が経過した際に呼ばれます
            // Called when time has elapsed without reading the barcode after starting to read it
            Android.Util.Log.Debug(TAG, "onTimeout");
        }

        public void OnDecodeStart()
        {
            //読み取り開始時に呼ばれます
            //Called at the start of a reading
            isScanning = true;
            Android.Util.Log.Debug(TAG, "onDecodeStart");
        }

        public void OnDecodeStop()
        {
            //読み取りの可否にかかわらず、読み取り終了時に呼ばれます
            // called at the end of a read, regardless of whether or not a read is possible
            isScanning = false;
            Android.Util.Log.Debug(TAG, "onDecodeStop");
        }

        public void OnConnect()
        {
            //スキャナとの接続が開始された際に呼ばれます
            //スキャナの設定を変更する場合、アプリを起動してからこのメソッドが呼ばれた後に変更してください。
            //Called when a connection to the scanner is initiated
            // If you want to change the scanner settings, do so after the application is launched and after this method is called.
            Android.Util.Log.Debug(TAG, "onConnect");

            //[設定変更]アプリとの接続が完了したらトリガキーを有効にする
            //[Change Settings] Activate the trigger key once the connection with the application is established.
            ScannerSettings settings = scanner.Settings;
            settings.SoftwareScanner.H35.ScanKeyIsEnable = true;
            scanner.SetSettings(settings);
        }

        public void OnDisconnect()
        {
            //スキャナとの接続が終了した際に呼ばれます
            //Called when the connection to the scanner is terminated
            Android.Util.Log.Debug(TAG, "onDisconnect");
        }

        public void OnImageData(Bitmap bitmap, byte[] byteArray, int i)
        {
            //スキャナから画像を取得した際に呼ばれます
            //Called when an image is acquired from the scanner
            Android.Util.Log.Debug(TAG, "onImageData");
            ImageView imageView = FindViewById<ImageView>(Resource.Id.imageView1);
            imageView.SetImageBitmap(bitmap);
        }


        public void SetButtonEvent()
        {
            var button = FindViewById<Button>(Resource.Id.buttonScan);
            button.Click += (s, e) =>
            {
                if (!scanner.IsConnected) return;
                if (isScanning)
                {
                    scanner.StopScan();
                }
                else
                {
                    scanner.StartScan();
                }
            };

            button = FindViewById<Button>(Resource.Id.buttonSettings);
            button.Click += (s, e) =>
            {
                //設定を変更する手順です
                //[Change Settings] CameraIllumination ON/OFF
                ScannerSettings settings = scanner.Settings;

                if (settings != null)
                {
                    if (settings.ReadOption.CameraIlluminationValue == ReadOption.CameraIllumination.TurnOn)
                    {
                        settings.ReadOption.CameraIlluminationValue = ReadOption.CameraIllumination.TurnOff;
                    }
                    else
                    {
                        settings.ReadOption.CameraIlluminationValue = ReadOption.CameraIllumination.TurnOn;
                    }

                }

                scanner.SetSettings(settings);
            };

            button = FindViewById<Button>(Resource.Id.buttonTakePicture);
            button.Click += (s, e) =>
            {
                if (!scanner.IsConnected) return;
                scanner.TakePicture(1);
            };
        }


    }
}